package com.bwie.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.domain.Admin;
import com.bwie.common.domain.request.Userlogin;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.feign.AuthFeign;
import com.bwie.service.AuthService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Random;
import java.util.concurrent.TimeUnit;

/**
 * @Author: zhx
 * @Package: com.bwie.service.impl
 * @Datetime: 2023/12/18 16:11
 * @Filename: AuthServiceImpl
 */
@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private AuthFeign authFeign;

    @Autowired
    private RedisTemplate<String,Object> redisTemplate;
    @Override
    public Result getCode(String phone) {

        Result<Admin> result = authFeign.findPhone(phone);
        Admin data = result.getData();

        if(data==null){
            return Result.error("用户不存在");
        }
        String code = "";
        for (int i = 0; i < 4; i++) {
            code+=new Random().nextInt(10);
        }

        System.out.println("生成的验证码为："+code);

        redisTemplate.opsForValue().set(phone,code,15, TimeUnit.MINUTES);

        return Result.success(code,"获取验证码成功");
    }

    @Override
    public Result login(Userlogin userlogin) {

        Result<Admin> result = authFeign.findPhone(userlogin.getPhone());
        Admin admin = result.getData();

        if(admin==null){
            return Result.error("用户不存在");
        }

        if(!redisTemplate.hasKey(userlogin.getPhone())){
            return Result.error("验证码过期");
        }

        Object o = redisTemplate.opsForValue().get(userlogin.getPhone());

        String o1 = (String) o;
        if(!userlogin.getCode().equals(redisTemplate.opsForValue().get(userlogin.getPhone()))) {
            return Result.error("验证码有误");
        }

        HashMap<String, Object> map = new HashMap<>();

        map.put(JwtConstants.USER_PHONE,userlogin.getPhone());

        String token = JwtUtils.createToken(map);

        admin.setToken(token);

        redisTemplate.opsForValue().set(JwtConstants.LOGIN_TOKEN+token, JSON.toJSONString(admin),30,TimeUnit.MINUTES);
        return Result.success(admin,"登录成功");
    }
}
