package com.bwie.gateway.filters;

import com.bwie.common.constants.JwtConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import io.jsonwebtoken.Claims;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;

@Component
public class AuthFilter implements GlobalFilter, Ordered {

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;


    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        //获取白名单
        List<String> whites = ignoreWhiteConfig.getWhites();

        //获取用户请求路径
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        System.out.println("路径>>>>>>"+path);

        //放行
        if (StringUtils.matches(path,whites)){
            return chain.filter(exchange);
        }

        //获取token
        String token = request.getHeaders().getFirst("token");
        if(token==null){
            return GatewayUtils.errorResponse(exchange,"必须携带token");
        }

        //token合法性
        Claims claims = null;
        try {
            claims = JwtUtils.parseToken(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange,"token不合法");
        }

        //时效性
        if(!redisTemplate.hasKey(JwtConstants.LOGIN_TOKEN+token)){
            return GatewayUtils.errorResponse(exchange,"token过期");
        }


        Long expire = redisTemplate.getExpire(JwtConstants.LOGIN_TOKEN+token);
//        if(expire<jwt原定过期时间的一半){
//              redisTemplate.expire("token:"+userId, 60, TimeUnit.SECONDS);
//        }
        //自动续期 15分钟
        redisTemplate.expire(JwtConstants.LOGIN_TOKEN+token,15, TimeUnit.MINUTES);
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
